@@ -51,6 +51,35 @@ try:
 except ImportError:
     pass
 
+###
+# Monkey-patch imaplib to support the ID command ([RFC 2971](https://www.rfc-editor.org/rfc/rfc2971)) - begin
+###
+imaplib.Commands['ID'] = ('AUTH', 'SELECTED')
+def imap_id_rfc2971(self, *kv_pairs, **kw):
+    """(typ, [data]) = <instance>.id(kv_pairs)
+    'kv_pairs' is a possibly empty list of keys and values.
+    'data' is a list of ID key value pairs or NIL.
+    NB: a single argument is assumed to be correctly formatted and is passed through unchanged
+    (for backward compatibility with earlier version).
+    Exchange information for problem analysis and determination.
+    The ID extension is defined in RFC 2971. """
+
+    name = 'ID'
+
+    if not kv_pairs:
+        data = 'NIL'
+    elif len(kv_pairs) == 1:
+        data = kv_pairs[0]     # Assume invoker passing correctly formatted string (back-compat)
+    else:
+        data = '(%s)' % ' '.join([(arg and self._quote(arg) or 'NIL') for arg in kv_pairs])
+
+    return self._simple_command(name, data, **kw)
+
+imaplib.IMAP4.id = imap_id_rfc2971
+###
+# Monkey-patch imaplib to support the ID command (RFC 2971) - end
+###
+
 try:
   import ssl
   SSLError = ssl.SSLError
@@ -492,8 +521,7 @@ class POP3_SSL_EXTENDED(poplib.POP3_SSL):
 
 #######################################
 class POP3SSLinitMixIn(CertMixIn):
-    '''Mix-In class to do POP3 over SSL initialization with Python 2.4's
-    poplib.POP3_SSL class.
+    '''Mix-In class to do POP3 over SSL with the poplib.POP3_SSL class.
     '''
     SSL = True
     def _connect(self):
@@ -506,8 +534,6 @@ class POP3SSLinitMixIn(CertMixIn):
         try:
             if ca_certs or ssl_version or ssl_ciphers:
                 using_extended_certs_interface = True
-                # Python 2.6 or higher required, use above class instead of
-                # vanilla stdlib one
                 msg = ''
                 if keyfile:
                     msg += 'with keyfile %s, certfile %s' % (keyfile, certfile)
@@ -603,14 +629,21 @@ class IMAP4_SSL_EXTENDED(imaplib.IMAP4_SSL):
        self.ssl_version = ssl_version
        self.ca_certs = ca_certs
        self.ssl_ciphers = ssl_ciphers
-       imaplib.IMAP4_SSL.__init__(self, host, port, keyfile, certfile)
+       self.keyfile = keyfile
+       self.certfile = certfile
+       try:
+           imaplib.IMAP4_SSL.__init__(self, host, port, keyfile, certfile)
+       except TypeError:
+           imaplib.IMAP4_SSL.__init__(self, host, port)
+
 
     def open(self, host='', port=imaplib.IMAP4_SSL_PORT, timeout=None):
        self.host = host
        self.port = port
        self.sock = socket.create_connection((host, port))
 
-       # Note timeout is available in python 3.9 in native imaplib's/ssl's open, but wrap_socket does not support it.  Keep it for the future.
+       # Note timeout is available in python 3.9 in native imaplib's/ssl's open,
+       # but wrap_socket does not support it.  Keep it for the future.
        self.timeout = timeout
 
        extra_args = { 'server_hostname': host }
@@ -631,7 +664,7 @@ class IMAP4_SSL_EXTENDED(imaplib.IMAP4_SSL):
 
 #######################################
 class IMAPSSLinitMixIn(CertMixIn):
-    '''Mix-In class to do IMAP over SSL initialization.
+    '''Mix-In class to do IMAP over SSL.
     '''
     SSL = True
     def _connect(self):
@@ -644,8 +677,6 @@ class IMAPSSLinitMixIn(CertMixIn):
         try:
             if ca_certs or ssl_version or ssl_ciphers:
                 using_extended_certs_interface = True
-                # Python 2.6 or higher required, use above class instead of
-                # vanilla stdlib one
                 msg = ''
                 if keyfile:
                     msg += 'with keyfile %s, certfile %s' % (keyfile, certfile)
@@ -756,8 +787,9 @@ class RetrieverSkeleton(ConfigurableBase):
                           octets) in a dictionary self.msgsizes, using the
                           message identifiers as keys.
 
-      _delmsgbyid(self, msgid) - delete a message from the message store based
-                                 on its message identifier.
+      _flagmsgbyid(self, msgid) - On default flag message msgid for deletion.
+                                  For IMAP imap_on_delete allows other flag:
+                                  return True if `Deleted` is among the flags.
 
       _getmsgbyid(self, msgid) - retrieve and return a message from the message
                                  store based on its message identifier.  The
@@ -795,6 +827,7 @@ class RetrieverSkeleton(ConfigurableBase):
         self._clear_state()
         self.conn = None
         self.supports_idle = False
+        self.supports_id = False
         ConfigurableBase.__init__(self, **args)
 
     def set_new_timestamp(self):
@@ -1005,8 +1038,9 @@ class RetrieverSkeleton(ConfigurableBase):
     def delmsg(self, msgid):
         if not self.__initialized:
             raise getmailOperationError('not initialized')
-        self._delmsgbyid(msgid)
-        self.deleted[msgid] = True
+        deleted = self._flagmsgbyid(msgid)
+        self.deleted[msgid] = deleted
+        return deleted
 
     def run_password_command(self):
         command = self.conf['password_command'][0]
@@ -1131,10 +1165,11 @@ class POP3RetrieverBase(RetrieverSkeleton):
             )
         self.gotmsglist = True
 
-    def _delmsgbyid(self, msgid):
+    def _flagmsgbyid(self, msgid):
         self.log.trace()
         msgnum = self._getmsgnumbyid(msgid)
         self.conn.dele(msgnum)
+        return True
 
     def _getmsgbyid(self, msgid):
         self.log.debug('msgid %s' % msgid + os.linesep)
@@ -1425,6 +1460,10 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         self.log.trace('got %s' % r + os.linesep)
         return r
 
+    def id(self):
+        server_id = self.conn.id('name', 'getmail', 'version', '6.0.0')
+        return server_id
+
     def list_mailboxes(self):
         '''List (selectable) IMAP folders in account.'''
         cmd = ('LIST', )
@@ -1437,7 +1476,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         # Close current mailbox so deleted mail is expunged.  One getmail
         # user had a buggy IMAP server that didn't do the automatic expunge,
         # so we do it explicitly here if we've deleted any messages.
-        if self.deleted:
+        if any(self.deleted):
             self.conn.expunge()
         self.write_oldmailfile(self.mailbox_selected)
         # And clear some state
@@ -1495,7 +1534,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
             self.mailbox_selected = mailbox
             # use *last* EXISTS returned
             count = int(count[-1])
-            uidvalidity = tostr(self.conn.response('UIDVALIDITY')[1][0])
+            uidvalidity = tostr(self.conn.response('UIDVALIDITY')[1][0] or b"") or None
         except imaplib.IMAP4.error as o:
             raise getmailOperationError('IMAP error (%s)' % o)
         except (IndexError, ValueError) as o:
@@ -1602,7 +1641,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
     def __getitem__(self, i):
         return self._mboxuidorder[i]
 
-    def _delmsgbyid(self, msgid):
+    def _flagmsgbyid(self, msgid):
         self.log.trace()
         try:
             uid = self._getmboxuidbymsgid(msgid)
@@ -1615,10 +1654,16 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                     'COPY', uid, self.conf['move_on_delete']
                 )
             self.log.debug('deleting message "%s"' % uid + os.linesep)
+
+            # On GMail you need to remove LABELS before deleting
+            if 'X-GM-EXT-1' in self.conn.capabilities:
+                self._parse_imapuidcmdresponse('STORE', uid,'X-GM-LABELS', '()')
+
+            flag = self.conf.get('imap_on_delete',None) or r'(\Deleted \Seen)'
             response = self._parse_imapuidcmdresponse(
-                'STORE', uid, 'FLAGS',
-                self.conf.get('imap_on_delete',None) or r'(\Deleted \Seen)'
+                'STORE', uid, 'FLAGS', flag
             )
+            return 'Deleted' in flag
         except imaplib.IMAP4.error as o:
             raise getmailOperationError('IMAP error (%s)' % o)
 
@@ -1716,7 +1761,7 @@ class IMAPRetrieverBase(RetrieverSkeleton):
         results = ext.groupdict()
         metadata = {}
         for item in ('LABELS', 'THRID', 'MSGID'):
-            if item in results and results[item]:
+            if results.get(item):
                 metadata['X-GMAIL-%s' % item] = results[item]
 
         return metadata
@@ -1816,6 +1861,12 @@ class IMAPRetrieverBase(RetrieverSkeleton):
                 self.supports_idle = True
                 imaplib.Commands['IDLE'] = ('AUTH', 'SELECTED')
 
+            if 'ID' in self.conn.capabilities:
+                self.supports_id = True
+
+            if self.supports_id and self.conf['imap_id_extension']:
+                self.id()
+
             if self.mailboxes == ('ALL', ):
                 # Special value meaning all mailboxes in account
                 self.mailboxes = tuple(self.list_mailboxes())
